#include "BabyDisplay.h"

const Int_t kNwires   = 10;
const Int_t kNpages   = 4;
const Int_t kNbuttons = 6;

ClassImp(BabyDisplay)

//---------------------------------------------------------------------------
BabyDisplay::BabyDisplay()
	: TGMainFrame(gClient->GetRoot(), 900, 800, kHorizontalFrame), 
	  fCanvas(0), fCont(0), fBottomButtons(0),
	  fStatusBar(0), fStatus(0),
	  fMenuBar(0), fMenus(0),
          fCounterInfo(0),
	  fPage(0),
	  fIsInitialized(kFALSE), fInLoop(kFALSE)
{
   fButtons = new TGTextButton* [kNbuttons];
   fButtons[0] = 0;
   fButtons[1] = 0;
   fButtons[2] = 0;
   fButtons[3] = 0;
   fButtons[4] = 0;
   fButtons[5] = 0;
}

//---------------------------------------------------------------------------
BabyDisplay::~BabyDisplay()
{
   if (fCont)   delete fCont;
   if (fCanvas) delete fCanvas;
}

//---------------------------------------------------------------------------
void BabyDisplay::Initialize()
{
    SetWindowName("Baby Chamber Event Display");

    TGLayoutHints *lhC 
    	= new TGLayoutHints(kLHintsCenterX,2,2,2,2);
    TGLayoutHints *lhTL
    	= new TGLayoutHints(kLHintsTop|kLHintsLeft|kLHintsExpandX,1,1,1,1);
    TGLayoutHints *lhTLnoexpand
    	= new TGLayoutHints(kLHintsTop|kLHintsLeft,1,1,1,1);
    TGLayoutHints *lhTRnoexpand
    	= new TGLayoutHints(kLHintsTop|kLHintsRight,1,1,1,1);
    TGLayoutHints *lhItem
    	= new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 1, 0, 0);

    fCont       = new TGCompositeFrame(this,900,700,kVerticalFrame);

// ------------------
//  Status Bar
// ------------------
    fStatusBar  = new TGCompositeFrame(fCont,100,5,kHorizontalFrame);

// ------------------
//  Status Bar Items
// ------------------
    fStatus     = new TGLabel(fStatusBar,"Initializing");
    Int_t nevt  = gJSF->GetEventNumber() + 1;
    if (nevt == 1 && gJSF->GetInput()) {
       if (gJSF->GetEvent(nevt)) {
          UpdateStatus();
    	  gJSF->Process();
       } else {	
          cerr << "Failed to get event #" << nevt << endl;
       }
    }
    fStatusBar->AddFrame(fStatus,lhTLnoexpand);

    fMenuBar    = new TGMenuBar(fStatusBar, 1, 1, kHorizontalFrame);
    fMenus      = new TGPopupMenu(fClient->GetRoot());
    fMenus->Associate(this);
    fMenus->AddEntry("New Canvas", 9001);		// Canvas
    fMenus->AddEntry("Open Browser", 9002);		// Browser
    fMenuBar->AddPopup("Tools", fMenus, lhItem);    
    fStatusBar->AddFrame(fMenuBar,lhTRnoexpand);

    fCounterInfo = new TGLabel(fStatusBar,"Initializing");
    if (nevt == 1 && gJSF->GetInput()) {
      if (gJSF->GetEvent(nevt)) {
	UpdateCounterInfo();
	gJSF->Process();
      } else {
	cerr << "Failed to get event #" << nevt << endl;
      }
    }
    fStatusBar->AddFrame(fCounterInfo,lhC);


    fCont->AddFrame(fStatusBar,lhTL);

// ------------------
//  Display Area
// ------------------
    fCanvas = new TRootEmbeddedCanvas("DisplayArea",fCont,800.,650.);
    fCont->AddFrame(fCanvas,lhC);

// ------------------
//  Bottom Buttons
// ------------------
    fBottomButtons = new TGCompositeFrame(fCont,10,10,kHorizontalFrame);
    fButtons[0] = new TGTextButton(fBottomButtons, "Page 1",8003);
    fButtons[1] = new TGTextButton(fBottomButtons, "Next",  8004);
    fButtons[2] = new TGTextButton(fBottomButtons, "Prev",  8005);
    fButtons[3] = new TGTextButton(fBottomButtons, "Go",    8006);
    fButtons[4] = new TGTextButton(fBottomButtons, "Print", 8007);
    fButtons[5] = new TGTextButton(fBottomButtons, "Quit",  8008);

    for (Int_t i=0; i<kNbuttons; i++) {
       fButtons[i]->Associate(this);
       fBottomButtons->AddFrame(fButtons[i],lhTL);
    }
    fCont->AddFrame(fBottomButtons,lhTL);

    AddFrame(fCont,lhC);
    fCont->Layout();
    fCanvasDirectory=gDirectory;

    MapSubwindows();
    Resize(GetDefaultSize());
    Layout();
    MapWindow();
    
    gROOT->LoadMacro("UserMacro.C");
    gROOT->ProcessLine("useranl->Initialize();");
    fIsInitialized = kTRUE;
    Display();
}

//---------------------------------------------------------------------------
void BabyDisplay::Display()
{
   if (!fIsInitialized) {
      Initialize();
   } else {
      fCanvas->GetCanvas()->cd();
      gPad->Clear();
      fCanvas->Clear();
   } 
   TDirectory *olddir = gDirectory;
   fCanvasDirectory->cd();

   fCanvas->GetCanvas()->cd();
   gROOT->ProcessLine("useranl->DrawHist();");

   if (fCanvasDirectory != olddir) olddir->cd();
}

//---------------------------------------------------------------------------
void BabyDisplay::ProcessButton(Long_t parm1)
{
   switch(parm1) {
      case 8008: { // Quit
         gJSF->Terminate();
         gJSF->GetOutput()->Write();
         CloseWindow();   // this also terminates theApp
         gROOT->ProcessLine(".q;");
         break;
      }
      case 8007: { // SaveAs
         Char_t fname[256];
         sprintf(fname,"run%4.4d.evt%5.5d.eps",
         		gJSF->GetRunNumber(), gJSF->GetEventNumber());
         gPad->SaveAs(fname);
         break;
      }
      case 8006: { // Go
         // cerr << "Go button is pressed" << endl;
         Int_t nevt = gJSF->GetEventNumber() + 1;
#if 1
	 cerr << "Event " << nevt << endl;
#endif
         while (gJSF->GetEvent(nevt++)) {
            fInLoop = kTRUE;
            cerr << "Event No " << nevt << endl;
            Display();
         }
         fInLoop = kFALSE;
	 gJSF->Process();
         UpdateStatus();
	 UpdateCounterInfo();
         Display();
         break;
      }
      case 8005: { // Prev
         // cerr << "Prev button is pressed" << endl;
         Int_t nevt = gJSF->GetEventNumber() - 1;
#if 1
	 cerr << "Event " << nevt << endl;
#endif
	 if (nevt+1 > 0) gJSF->Clear();
         if (gJSF->GetInput()) {
            if (gJSF->GetEvent(nevt)) {
	      UpdateStatus();
	      UpdateCounterInfo();
	    }
            else	    cerr << "Failed to get event #" << nevt << endl;
         }
         gJSF->Process();
         Display();
         break;
      }
      case 8004: { // Next
         // cerr << "Next button is pressed" << endl;
         Int_t nevt = gJSF->GetEventNumber() + 1;
#if 1
	 cerr << "Event " << nevt << endl;
#endif
	 if (nevt-1 > 0) gJSF->Clear();
         if (gJSF->GetInput()) {
            if (gJSF->GetEvent(nevt)) {
	      UpdateStatus();
	      UpdateCounterInfo();
	    }
            else	    cerr << "Failed to get event #" << nevt << endl;
         }
         gJSF->Process();
         Display();
         break;
      }
      case 8003: { // Page
         fPage++;
         fPage = fPage % kNpages;
         Char_t label[256];
         sprintf(label, "Page %i", fPage+1);
         fButtons[0]->SetText(new TGHotString(label));
         // cerr << "Switch to Page " << fPage << endl;
         Display();
         break;
      }
      default:
         break;
   }
}

//---------------------------------------------------------------------------
Bool_t BabyDisplay::ProcessMessage(Long_t msg, Long_t parm1, Long_t)
{
   switch (GET_MSG(msg)) {
      case kC_COMMAND:
         switch (GET_SUBMSG(msg)) {
            case kCM_BUTTON:
                if (parm1 >= 8000) ProcessButton(parm1);
                break;
            case kCM_MENUSELECT:
               break;
            case kCM_MENU:
               switch (parm1) {
                  case 9001: {
                     TCanvas *c;
                     c = new TCanvas();
                     break;
                  }
                  case 9002: {
                     TBrowser *b;
                     b = new TBrowser();
                     break;
                  }
                  default:
                     break;
               }
            default:
               break;
         }
         break;
      default:
         break;
   }
   return kTRUE;
}

//---------------------------------------------------------------------------
void BabyDisplay::UpdateStatus()
{
    //Char_t status[256];
    Char_t *status = new Char_t [256];
    sprintf(status,"Run %4.4d  Event %5.5d",
         		gJSF->GetRunNumber(), gJSF->GetEventNumber());
    fStatus->SetText(new TGString(status));
}

//---------------------------------------------------------------------------
void BabyDisplay::UpdateCounterInfo()
{
  CDCRawData     *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
  CDCRawDataBuf  *evt = (CDCRawDataBuf *)raw->EventBuf();
  CDCBeginRunBuf *br  = raw->GetBeginRunBuf();

  Int_t              ncdc = evt->GetNADCData();
  TClonesArray   *adcdata = evt->GetADCData();

  Int_t i;
  Int_t tc1adc = 0;
  Int_t tc2adc = 0;
  Int_t hod1sumadc = 0;
  Int_t csiadc = 0;
  for (i = 0; i < ncdc; i++) {
    CDCRawADCDatum *adcdatum = (CDCRawADCDatum *)adcdata->UncheckedAt(i);
    Int_t st = adcdatum->GetStationNo();
    if (st != 20) continue;
    Int_t a;
    Int_t id;
    Int_t cr = 0;               // We have only 1 crate now.
    Int_t ch = adcdatum->GetChannelNo();
    br->EChannelToDChannel(cr,st,ch,id,a);
    Double_t adc  = adcdatum->GetADC();

    switch (a) {
    case 31: {
      tc1adc = (Int_t)adc;
      //      cout << "ADC of TC1 = " << tc1adc << endl;
      break;
    }
    case 33: {
      tc2adc = (Int_t)adc;
      //      cout << "ADC of TC2 = " << tc2adc << endl;
      break;
    }
    case 34: {
      hod1sumadc = (Int_t)adc;
      //      cout << "ADC of HOD1 Sum = " << hod1sumadc << endl;
      break;
    }
    case 35: {
      csiadc = (Int_t)adc;
      //      cout << "ADC of CsI = " << csiadc << endl;
      break;
    }
    default:
      break;
    }
  }
  Char_t *cntinfo = new Char_t [256];
  if (gJSF->GetRunNumber() < 858) {
    sprintf(cntinfo,"Counter ADC : TC1 = %4.4d,  TC2 = %4.4d,  CsI = %4.4d",
	    tc1adc, tc2adc, csiadc);
  } else {
    sprintf(cntinfo,"Counter ADC : TC1 = %4.4d,  HOD1Sum = %4.4d,  TC2 = %4.4d,  CsI = %4.4d",
	    tc1adc, hod1sumadc, tc2adc, csiadc);
  }
  TGString *str = new TGString(cntinfo);
  fCounterInfo->SetText(str);
}

//---------------------------------------------------------------------------
Int_t BabyDisplay::SkipChamberData()
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   CDCRawData     *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
   CDCRawDataBuf  *evt = (CDCRawDataBuf *)raw->EventBuf();

   Int_t              ncdc = evt->GetNADCData();
   TClonesArray   *adcdata = evt->GetADCData();
   Int_t i;
   for (i = 0; i < ncdc; i++) {
      CDCRawADCDatum *adcdatum = (CDCRawADCDatum *)adcdata->UncheckedAt(i);
      Int_t st = adcdatum->GetStationNo();
      if (st > 13) break;
   }
   last->cd();
   return i;
}
